#! /usr/bin/perl -w

# Remove the receipts of a TcpPep installation so a subsequent
# installation of Tcp Pep release will work correctly.

### This script will remove

#  1. The package receipts for TcpPepSw.pkg, tcppep.pkg, TcpPepMeta.pkg. 

use strict;
use File::Basename;
use Socket;

use vars qw ($do_nothing $print_donothing_removals $receipts_dir $verbose $noisy_warnings $debug);
use vars qw ($suppress_spin $spin_counter $spin_state $spin_slower_downer);


# The only setting that most people would ever set is $do_nothing = 1,
# which will make the script print a list of files/directories/receipts
# it would remove, instead of doing anything.

$do_nothing = 0;
$print_donothing_removals = 1;
$verbose = 1;
$noisy_warnings = 0;
$debug = 0;

$suppress_spin = 0;
$spin_counter = 0;
$spin_state = 0;
spin_rate_slow ();

# The single rm -rf in this script uses $receipts_dir -- change with care.
$receipts_dir = "/Library/Receipts";

$| = 1;
if ($do_nothing == 0 && $< != 0)
  {
    die "ERROR: Must be run with root permissions--prefix command with 'sudo'.\n";
  }

sub main
  {
    sendShutDown();
    remove_generated_files();
    remove_main_packages ();
    remove_generated_directories ();

    pre_print ();
    print "Finished uninstalling.\n";
  }

sub sendShutDown
{
my($proto, $remote, $local, $myport, $rport, $raddr, $iaddr, $paddr, $hispaddr, $msg) ;

$local = 'localhost' ;
$myport = 33333 ; #STUB_PORT
$iaddr = inet_aton($local) ;
$paddr = sockaddr_in($myport, $iaddr) ;

$remote = 'localhost' ;
$rport = 16008 ; #MON_PORT
$raddr = inet_aton($remote) ;
$hispaddr = sockaddr_in($rport, $raddr) ;

$proto = getprotobyname("udp") ;

socket(SOCK,PF_INET, SOCK_DGRAM, $proto) || die "Socket: $!" ;
bind (SOCK, $paddr) ;

$msg = pack('CC',88,60) ;
send(SOCK, $msg,0, $hispaddr) || die "Send:$!\n" ;
shutdown(SOCK, 2) ;
}

sub remove_main_packages
  {

    my @pkglist = ('tcppep',
                   'TcpPepSw');

    foreach (@pkglist)
      {
        my $pkgname = $_;
        my $pkgname_fullpath = "$receipts_dir/$pkgname.pkg";
        my $pkgname_bom_fullpath = undef;

        print_verbose ("REMOVING PACKAGE $pkgname.pkg\n");

        next if (! -d "$pkgname_fullpath" );

        foreach my $f ("$pkgname_fullpath/Contents/Resources/$pkgname.bom",
                       "$pkgname_fullpath/Contents/Archive.bom",
                       "$pkgname_fullpath/Contents/Resources/Archive.bom") {
            if (-f $f && -r $f) {
                $pkgname_bom_fullpath = $f;
                last;
            }
        }

        next if (!defined ($pkgname_bom_fullpath));

#
# First, get list of files in BOM, remove them
#
        spin_rate_slow ();

        open (LSBOM, "/usr/bin/lsbom -l -f -p f '$pkgname_bom_fullpath' |") or next;
        while (<LSBOM>)
          {
            chomp;
            m#^\.(/.*)$#;
            my $filename = $1;
            next if (!defined ($filename) || $filename eq "");

            remove_a_file ($filename);
          }
        close (LSBOM);

#
# Second, get list of directories in BOM, remove them.
#
# Directories are only removed if they're empty.  Directories are only
# empty when all their subdirectories are removed.  So we need to build
# an internal representation of the directory hierarchy and remove it
# depth-first.
#
        my $rooth = { };

        open (LSBOM, "/usr/bin/lsbom -d -p f '$pkgname_bom_fullpath' |") or next;
        while (<LSBOM>)
          {
            chomp;
            m#^\.(/.*)$#;
            my $directory = $1;
            next if (!defined ($directory) || $directory eq "");
            if (-d $directory)
              {
                $rooth = add_directory_to_tree ($directory, $rooth);
              }
            else
              {
                if ($noisy_warnings)
                  {
                    print_warning ("Warning: \"$directory\" listed in BOM but not present on system.\n");
                  }
              }
          }
        close (LSBOM);

        spin_rate_fast ();

#
# Third, remove package receipts from the system.
#
        remove_package_receipts ("$pkgname.pkg");
      }
      
  }

sub remove_generated_files
  {
    
    my $pepdir ='/Applications/tcppep';
    my $startupitem= '/System/Library/StartupItems/TcpPep';

    my @files_to_remove = 
      (
        "$pepdir/itcp",
        "$pepdir/tcpmon",
        "$pepdir/tsnconfig",
        "$pepdir/tsnLocalconfig",
        "$pepdir/tcpmonConfig",
        "$pepdir/firewall.sh",
        "$pepdir/bganMMIStub",
        "$pepdir/out",
        "$pepdir/TcpPepLog",
        "$startupitem/TcpPep",
        "$startupitem/StartupParameters.plist",
      );
    system("rm -rf /Applications/tcppep/\"TCP-PEP GUI.app\"");
    system("rm /Applications/tcppep/loginwindow.plist");
   foreach (@files_to_remove)
     {
       remove_a_file ($_);
     }

  }

sub remove_generated_directories
  {
    my $rooth = { };

    $rooth->{"/"}->{"Applications"}->{"tcppep"} = {};
    $rooth->{"/"}->{"System"}->{"Library"}->{"StartupItems"}->{"TcpPep"} = {};
    remove_empty_directories ($rooth, "/");
  }

sub remove_a_file
  {
    my $fn = shift;
    my $dirname = dirname ($fn);
    my $basename = basename ($fn);
    my $ufs_rsrc_file = "$dirname/._$basename";

    progress_point ();
    return if (!defined ($fn) || $fn eq "");

# This is a shared file with important copyright information in it.
    if ($fn eq "/Library/Documentation/Acknowledgements.rtf" ||
        $fn =~ m#^/usr/share/terminfo/#)
      {
        if ($noisy_warnings)
          {
            print_warning ("Warning: file \"$fn\" intentionally not removed, even though it's in the BOM.\n");
          }
        return;
      }

    if (! -f $fn && ! -l $fn)
      {
        if ($noisy_warnings)
          {
            print_warning ("Warning: file \"$fn\" not found on disc.  And I was so looking forward to removing it.\n");
          }
        return;
      }

    if ($do_nothing == 1) 
      {
        print_donothing ("rm $fn\n");
        print_donothing ("rm $ufs_rsrc_file\n") if ( -f $ufs_rsrc_file);
      }
    else
      {
        if ((unlink $fn) != 1)
          {
            print_error ("ERROR: Encountered error while trying to remove \"$fn\"\n");
          }
        if (-f $ufs_rsrc_file && ((unlink ($ufs_rsrc_file)) != 1))
          {
            print_error ("ERROR: Encountered error while trying to remove \"$ufs_rsrc_file\"\n");
          }
      }
  }

sub remove_a_dir
  {
    my $dir = shift;

    progress_point ();
    return if (!defined ($dir) || $dir eq "" || $dir eq "/" || $dir eq "/usr");
    if (! -d $dir)
      {
        if ($noisy_warnings)
          {
            print_warning ("Warning: directory \"$dir\" not found on disc.  And I was so looking forward to removing it.\n");
          }
        return;
      }

    if ($do_nothing == 1) 
      {
        print_donothing ("rmdir $dir\n");
      }
    else
      {
        rmdir $dir;
      }
  }
sub remove_empty_directories
  {
    my $rooth = shift;
    my $path = shift;
    my $children = (scalar (keys %{$rooth}));
    my $dirs_remain = 0;

    if ($children > 0)
      {
        foreach my $dirname (sort keys %{$rooth})
          {
            my $printpath;
            $printpath = "$path/$dirname";
            $printpath =~ s#^/*#/#;
            remove_empty_directories ($rooth->{$dirname}, "$printpath");
            $dirs_remain = 1 if (-d "$printpath");
          }
      }
sub maybe_remove_ds_store
  {
    my $path = shift;
    my $filecount = 0;
    return if (!defined ($path) || $path eq "" || $path eq "/" || $path eq "/usr");
    return if (! -f "$path/.DS_Store");

    open (LS, "/bin/ls -a '$path' |");
    while (<LS>)
      {
        chomp;
        next if (m#^\.$# || m#^\.\.$#);
        $filecount++;
      }
    close (LS);

    if ($filecount == 1 && -f "$path/.DS_Store")
      {
        remove_a_file ("$path/.DS_Store");
      }
  }

# If we've removed all the children, and there's a .DS_Store file, remove
# it if it's last file sitting around.
     if ($dirs_remain == 0)
       {
         maybe_remove_ds_store ("$path");
       }

     remove_a_dir ("$path");
  }
sub remove_package_receipts
  {
    my $pkgname = shift;
    $pkgname =~ s#/##g;  # There shouldn't be any path seps in the pkg name...
    return if (!defined ($pkgname) || $pkgname eq "" 
               || $pkgname eq "." || $pkgname eq "..");

    my $pkgdir = "$receipts_dir/$pkgname";
    return if (!defined ($pkgdir) || $pkgdir eq "" || ! -d $pkgdir);

    my @rmcmd = ('/bin/rm', '-rf', "$pkgdir");
    if ($do_nothing == 1)
      {
        print_donothing ("rm -rf $pkgdir\n");
      }
    else
      {
        system @rmcmd;
        my $retcode = $? >> 8;
        if ($retcode != 0)
          {
            print_warning ("Warning:  There may have been a problem removing \"$pkgdir\"\n");
          }
      }
  }


sub print_donothing
  {
    my $msg = shift;
    pre_print ();
    return if ($print_donothing_removals != 1);
    print $msg;
  }

sub print_verbose
  {
    my $msg = shift;
    return if ($verbose != 1);
    pre_print ();
    print $msg;
  }

sub print_warning
  {
    my $msg = shift;
    pre_print ();
    print STDERR $msg;
  }

sub print_error
  {
    my $msg = shift;
    pre_print ();
    print STDERR $msg;
  }

sub print_debug
  {
    my $msg = shift;
    pre_print ();
    print $msg;
  }

sub pre_print 
  {
    print " " if ($suppress_spin != 0);
  }

sub spin_rate_slow
  {
    $spin_slower_downer = 150;
  }

sub spin_rate_fast
  {
    $spin_slower_downer = 75;
  }

sub progress_point
  {
    return if ($suppress_spin != 0);
    $spin_counter++;
    if (($spin_counter % $spin_slower_downer) == 0)
      {
        my $spin_chars = "|/-\\";
        my $c = substr ($spin_chars, $spin_state % 4, 1);
        $spin_state++;
        print "[7m$c[m";
      }
  }

main ();
